@@ -27,6 +27,8 @@
 
 #define HANDLE void*
 
+class CConfigManager;
+
 struct SWorkspaceRule {
     std::string                        monitor         = "";
     std::string                        workspaceString = "";
@@ -36,6 +38,7 @@ struct SWorkspaceRule {
     bool                               isPersistent    = false;
     std::optional<CCssGapData>         gapsIn;
     std::optional<CCssGapData>         gapsOut;
+    std::optional<CCssGapData>         floatGaps = gapsOut;
     std::optional<int64_t>             borderSize;
     std::optional<bool>                decorate;
     std::optional<bool>                noRounding;
@@ -141,8 +144,18 @@ struct SFirstExecRequest {
 struct SFloatCache {
     size_t hash;
 
-    SFloatCache(PHLWINDOW window) {
-        hash = std::hash<std::string>{}(window->m_szClass) ^ (std::hash<std::string>{}(window->m_szTitle) << 1);
+    SFloatCache(PHLWINDOW window, bool initial) {
+        // Base hash from class/title
+        size_t baseHash = initial ? (std::hash<std::string>{}(window->m_initialClass) ^ (std::hash<std::string>{}(window->m_initialTitle) << 1)) :
+                                    (std::hash<std::string>{}(window->m_class) ^ (std::hash<std::string>{}(window->m_title) << 1));
+
+        // Use empty string as default tag value
+        std::string tagValue = "";
+        if (auto xdgTag = window->xdgTag())
+            tagValue = xdgTag.value();
+
+        // Combine hashes
+        hash = baseHash ^ (std::hash<std::string>{}(tagValue) << 2);
     }
 
     bool operator==(const SFloatCache& other) const {
@@ -159,6 +172,32 @@ namespace std {
     };
 }
 
+class CMonitorRuleParser {
+  public:
+    CMonitorRuleParser(const std::string& name);
+
+    const std::string&         name();
+    SMonitorRule&              rule();
+    std::optional<std::string> getError();
+    bool                       parseMode(const std::string& value);
+    bool                       parsePosition(const std::string& value, bool isFirst = false);
+    bool                       parseScale(const std::string& value);
+    bool                       parseTransform(const std::string& value);
+    bool                       parseBitdepth(const std::string& value);
+    bool                       parseCM(const std::string& value);
+    bool                       parseSDRBrightness(const std::string& value);
+    bool                       parseSDRSaturation(const std::string& value);
+    bool                       parseVRR(const std::string& value);
+
+    void                       setDisabled();
+    void                       setMirror(const std::string& value);
+    bool                       setReserved(const SMonitorAdditionalReservedArea& value);
+
+  private:
+    SMonitorRule m_rule;
+    std::string  m_error = "";
+};
+
 class CConfigManager {
   public:
     CConfigManager();
@@ -244,48 +283,52 @@ class CConfigManager {
     std::optional<std::string> handleBindWS(const std::string&, const std::string&);
     std::optional<std::string> handleEnv(const std::string&, const std::string&);
     std::optional<std::string> handlePlugin(const std::string&, const std::string&);
+    std::optional<std::string> handlePermission(const std::string&, const std::string&);
 
-    std::string                configCurrentPath;
+    std::optional<std::string> handleMonitorv2(const std::string& output);
+    Hyprlang::CParseResult     handleMonitorv2();
 
-    bool                       m_bWantsMonitorReload                  = false;
-    bool                       m_bNoMonitorReload                     = false;
-    bool                       isLaunchingExecOnce                    = false; // For exec-once to skip initial ws tracking
-    bool                       m_bLastConfigVerificationWasSuccessful = true;
+    std::string                m_configCurrentPath;
+
+    bool                       m_wantsMonitorReload                  = false;
+    bool                       m_noMonitorReload                     = false;
+    bool                       m_isLaunchingExecOnce                 = false; // For exec-once to skip initial ws tracking
+    bool                       m_lastConfigVerificationWasSuccessful = true;
 
     void                       storeFloatingSize(PHLWINDOW window, const Vector2D& size);
     std::optional<Vector2D>    getStoredFloatingSize(PHLWINDOW window);
 
   private:
-    UP<Hyprlang::CConfig>                            m_pConfig;
+    UP<Hyprlang::CConfig>                            m_config;
 
     std::vector<std::string>                         m_configPaths;
 
-    Hyprutils::Animation::CAnimationConfigTree       m_AnimationTree;
+    Hyprutils::Animation::CAnimationConfigTree       m_animationTree;
 
-    std::string                                      m_szCurrentSubmap = ""; // For storing the current keybind submap
+    std::string                                      m_currentSubmap = ""; // For storing the current keybind submap
 
-    std::vector<SExecRequestedRule>                  execRequestedRules; // rules requested with exec, e.g. [workspace 2] kitty
+    std::vector<SExecRequestedRule>                  m_execRequestedRules; // rules requested with exec, e.g. [workspace 2] kitty
 
-    std::vector<std::string>                         m_vDeclaredPlugins;
-    std::vector<SPluginKeyword>                      pluginKeywords;
-    std::vector<SPluginVariable>                     pluginVariables;
+    std::vector<std::string>                         m_declaredPlugins;
+    std::vector<SPluginKeyword>                      m_pluginKeywords;
+    std::vector<SPluginVariable>                     m_pluginVariables;
 
-    bool                                             isFirstLaunch = true; // For exec-once
+    bool                                             m_isFirstLaunch = true; // For exec-once
 
-    std::vector<SMonitorRule>                        m_vMonitorRules;
-    std::vector<SWorkspaceRule>                      m_vWorkspaceRules;
-    std::vector<SP<CWindowRule>>                     m_vWindowRules;
-    std::vector<SP<CLayerRule>>                      m_vLayerRules;
-    std::vector<std::string>                         m_dBlurLSNamespaces;
+    std::vector<SMonitorRule>                        m_monitorRules;
+    std::vector<SWorkspaceRule>                      m_workspaceRules;
+    std::vector<SP<CWindowRule>>                     m_windowRules;
+    std::vector<SP<CLayerRule>>                      m_layerRules;
+    std::vector<std::string>                         m_blurLSNamespaces;
 
-    bool                                             firstExecDispatched     = false;
-    bool                                             m_bManualCrashInitiated = false;
+    bool                                             m_firstExecDispatched  = false;
+    bool                                             m_manualCrashInitiated = false;
 
-    std::vector<SFirstExecRequest>                   firstExecRequests; // bool is for if with rules
-    std::vector<std::string>                         finalExecRequests;
+    std::vector<SFirstExecRequest>                   m_firstExecRequests; // bool is for if with rules
+    std::vector<std::string>                         m_finalExecRequests;
 
-    std::vector<std::pair<std::string, std::string>> m_vFailedPluginConfigValues; // for plugin values of unloaded plugins
-    std::string                                      m_szConfigErrors = "";
+    std::vector<std::pair<std::string, std::string>> m_failedPluginConfigValues; // for plugin values of unloaded plugins
+    std::string                                      m_configErrors = "";
 
     uint32_t                                         m_configValueNumber = 0;
 
@@ -295,6 +338,7 @@ class CConfigManager {
     std::optional<std::string>                resetHLConfig();
     std::optional<std::string>                generateConfig(std::string configPath);
     std::optional<std::string>                verifyConfigExists();
+
     void                                      postConfigReload(const Hyprlang::CParseResult& result);
     SWorkspaceRule                            mergeWorkspaceRules(const SWorkspaceRule&, const SWorkspaceRule&);
 
@@ -307,6 +351,7 @@ class CConfigManager {
     std::unordered_map<SFloatCache, Vector2D> m_mStoredFloatingSizes;
 
     friend struct SConfigOptionDescription;
+    friend class CMonitorRuleParser;
 };
 
 inline UP<CConfigManager> g_pConfigManager;
